import cuffeo from 'cuffeo';
import handranker from './inc/handranker.mjs';
import cfg from '../config.json' assert { type: 'json' };

const bot = await new cuffeo(cfg.clients);

const suits = { "♠": "S", "♣": "C", "♦": "D", "♥": "H" };

const stripColors = msg => msg.replace(/\x03\d{0,2}(,\d{0,2}|\x02\x02)?/g, '');
const rand = (max = 1) => ~~(Math.random() * (max - 1) + 1);

const parseCards = msg => {
  const output = [];
  for(const c of stripColors(msg).match(/(\w+)([♠♣️♦♥️️])/g)) {
    if(c.length === 3)
      output.push(`T${suits[c[2]]}`);
    else
      output.push(`${c[0]}${suits[c[1]]}`);
  }
  return output;
};

const env = {
  gamestate: 'preflop', // [preflop,flop,turn,river]
  hand: false,
  board: false,
  winchance: false,
  joined: false,
  callamount: 0,
  pot: 0,
  bank: 0,
  lastaction: false // debug purpose
};

bot.on("notice", msg => {
  if(msg.match(/^Your hand/)) {
    env.hand = parseCards(msg);
    console.log('hand:', env.hand);
  }
});

bot.on("message", e => {
  if(e.channel !== '#poker') {
    return;
  }
  if(e.message.match(new RegExp(`^${e.self.me.nickname}: Your bank account`))) {
    env.bank = +e.message.match(/is: (\d+) \(/)[1];
  }
  if(e.message === ".pj" || e.message === "pj" && !env.hand && !env.joined) {
    env.joined = true;
    //return e.reply("pj");
    return e.reply("bb\npj");
  }
  if(e.message === ".pl" && !env.hand) {
    env.joined = false;
    return e.reply("pl");
  }
  if(e.message === ".bb" && !env.hand)
    return e.reply("bb");
  if(e.message === ".bl" && !env.hand)
    return e.reply("bank loan");
  if(e.message === ".s" && env.hand)
    return e.reply("s");
  if(e.message === ".f")
    return e.reply("f");
  if(e.message === ".d" && !env.hand)
    return e.reply("d");

  if(e.user.nick !== 'hirc' || !env.hand)
    return;
  
  // tracker
  if(e.message.includes('raises the pot')) {
    env.callamount = +e.message.match(/raises the pot by (\d+)/)[1];
    env.pot += env.callamount;
    if(e.message.includes(e.self.me.nickname))
      env.bank -= env.callamount;
    return;
  }
  if(e.message.includes('calls')) {
    env.callamount = +e.message.match(/calls (\d+)\./)[1];
    env.pot += env.callamount;
    if(e.message.includes(e.self.me.nickname))
      env.bank -= env.callamount;
    return;
  }
  if(e.message.endsWith('blind).')) { // blinds
    env.callamount = +e.message.match(/pays (\d+) \(/)[1];
    env.pot += env.callamount;
    env.gamestate = 'preflop';
    if(e.message.includes(e.self.me.nickname))
      env.bank -= env.callamount;
    return;
  }
  
  // bot's turn
  if(e.message.match(new RegExp(`Current player: ${e.self.me.nickname}( |$)`))) {
    let action = 'c'; // default
    if(env.gamestate === 'preflop' || !env.winchance) { // preflop
      env.lastaction = action;
      return e.reply(action); // checkcall
    }

    if(e.message.endsWith('to call)')) { // callphase
      env.callamount = +e.message.match(/\((\d+) to call\)/)[1];
      
      if(env.winchance < 4300) {
        if(rand(5) === 1 && env.callamount < 80) { // bad hand, call anyway
          action = 'c';
        }
        else { // bad hand, fold
          action = 'f';
        }
      }
      else if(env.winchance > 7000) { // decent hand, raise
        if(rand(5) === 1) { // 20%
          action = 'r ' + (env.callamount + rand(5) * 10);
        }
      }
      else if(env.winchance > 15000) { // good hand lol
        if(rand(2) === 1) { // 50%
          action = 'r ' + (env.callamount + rand(6) * 10);
        }
      }
      else if(env.winchance > 20000) { // fuck them all
        action = 'r ' + (env.callamount + rand(7) * 10);
      }
    }
    else { // checkphase
      if(env.winchance > 7000) { // decend hand, raise
        if(rand(5) === 1) { // 20%
          action = 'r ' + (env.callamount + rand(5) * 10);
        }
      }
      else if(env.winchance > 15000) { // good hand lol
        if(rand(2) === 1) { // 50%
          action = 'r ' + (env.callamount + rand(6) * 10);
        }
      }
      else if(env.winchance > 20000) { // fuck them all
        action = 'r ' + (env.callamount + rand(7) * 10);
      }
    }

    if(action) {
      env.lastaction = action;
      return e.reply(action);
    }
  }

  // gamestate & board changes
  if(e.message.match(/^(Flop|Turn|River)/)) {
    env.board = parseCards(e.message);
    env.gamestate = e.message.match(/(\w+): /)[1].toLowerCase();

    const rank = handranker.evalHand([...env.board, ...env.hand]);
    env.winchance = rank.value;
  }

  // end of game
  if(e.message.includes('wins the pot') || e.message.includes('Split pot')) {
    if(e.message.includes(e.self.me.nickname)) {
      const amount = +e.message.match(/size (\d+) /)[1];
      if(e.message.includes('Split pot'))
        env.bank += ~~(amount / 2);
      else
        env.bank += amount;
    }

    env.gamestate = 'preflop';
    env.hand = false;
    env.board = false;
    env.winchance = false;
    env.joined = false;
    env.pot = 0;
    env.lastaction = false;
    return;
  }

  console.log(env);
});
